#!/usr/bin/perl

# Copyright 2007, Fonality Inc. ALL RIGHTS RESERVED.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Author: Adrian Otto, Samy Kamkar <support@fonality.com>
# Description: Shell script to enable local system configuration


use Term::ReadLine;
use Getopt::Long;
use LWP::UserAgent;
use Sys::Hostname qw(hostname);
use Digest::MD5 qw(md5_hex);
use File::Copy qw(copy);

my $stype = 'PBXtra';
open(VPN, "</etc/sysconfig/vpn");
while (<VPN>)
{
	if (/^GW.*trixbox/)
	{
		$stype = 'trixbox Pro';
		last;
	}
}
close(VPN);

my $interactive = 0;
GetOptions ("i" => \$interactive);

my $hostname = `/bin/hostname`;
chomp($hostname);

my $tty = `/usr/bin/tty`;
chomp($tty);

my $bup_status = _get_bup_status();

my $RUNNING_LOCALLY = 0;
if ($tty =~ /^\/dev\/tty[1-6]$/)
{
	$RUNNING_LOCALLY = 1;
}
else
{
	if(! $interactive)
	{
		print STDERR "ERROR: Unable to run remotely without -i argument.\n";
		die "USAGE: $0 -i\n";
	}
}

my $term = new Term::ReadLine "Network Troubleshooter";
main_sub($term);
exit(0);

##############################################################################
# show_menu
##############################################################################
sub show_menu
{
	print "\n";
	print " 1. Set IP Address of this system\n";
	print " 2. Show IP Address of this system\n";
	print " 3. Run the PING utility\n";
	print " 4. Run the TRACEROUTE utility\n";
	print " 5. Display the DNS Nameserver configuration\n";
	print " 6. Display the route table\n";
	print " 7. Display the ARP table\n";
	print " 8. Show Analog Phone Line voltage levels\n";
	print " 9. Show T1 Status\n";
	print "10. Show Active Calls\n";
	print "11. Set 'root' password\n";
	print "12. Reboot system\n";
	if ($stype ne 'PBXtra')
	{
		print "13. Re-scan for PSTN cards\n";
		print "14. Scan for new phones\n";
	}
	if ($bup_status->{is_backup})
	{
		print "\nP. Turn this server into the Primary Server\n";
	}
	if ($bup_status->{has_backup})
	{
		print "\nB. Turn this server into the Backup Server\n";
	}
	print "\n";
	print "Q. Quit and exit this tool\n";
	print "\n";
}


##############################################################################
# main_sub
##############################################################################
sub main_sub
{
	my $term = shift;
	print "\nNetwork Config/Troubleshooter\n";
	while(1) {
		# Before we show the menu, get the bup status in case it changed.
		$bup_status = _get_bup_status();
		show_menu();

		my $ans = _get_info_from_terminal($term,"Selection");
		chomp($ans);

		# After we've gotten the response, reload bup status in case they kept one screen up for a long time
		$bup_status = _get_bup_status();

		if($ans =~ /^1$/) {
			system('perl /var/adm/bin/ip_set.pl -i');
		}
		elsif($ans =~ /^2/) {
			show_ip($term);
		}
		elsif($ans =~ /^3/) {
			run_ping($term);
		}
		elsif($ans =~ /^4/) {
			run_traceroute($term);
		}
		elsif($ans =~ /^5/) {
			show_resolv_conf($term);
		}
		elsif($ans =~ /^6/) {
			show_route($term);
		}
		elsif($ans =~ /^7/) {
			show_arp($term);
		}
		elsif($ans =~ /^8/) {
			_show_voltage($term);
		}
		elsif($ans =~ /^9/) {
			_show_t1_status($term);
		}
		elsif($ans =~ /^10$/) {
			system('/usr/sbin/asterisk -r -x "show channels"');
		}
		elsif($ans =~ /^11$/) {
			_set_root_password($term);
		}
		elsif ($ans == 12)
		{
			print "Are you sure you want to reboot? [y/n]: ";
			chomp(my $answer = <STDIN>);
			if ($answer =~ /^y/)
			{
				system("/sbin/reboot");
			}
		}
		elsif ($ans == 13 && $stype ne 'PBXtra')
		{
			print "This option will erase and recreate all PSTN card configurations.\n";
			print "Are you sure you want to re-detect PSTN cards? [y/n]: ";
			chomp(my $answer = <STDIN>);
			if ($answer =~ /^y/)
			{
				system('/var/adm/bin/detect_pstn_cards.pl -c');
				system('/etc/init.d/asterisk start');
			}
		}
		elsif ($ans == 14 && $stype ne 'PBXtra')
		{
			system("/var/adm/bin/add_devices");
		}
		elsif ($bup_status->{is_backup} && $ans =~ /^[pP]/)
		{
			primarify($term);
		}
		elsif ($bup_status->{has_backup} && $ans =~ /^[bB]/)
		{
			backupify($term);
		}
		elsif($ans =~ /^sh/i)
		{
			$ENV{'HOME'} = '/root';
			chdir('/root');
			exec("/bin/bash");
		}
		elsif($ans =~ /^q/i)
		{
			print "Quitting\n";
			exit(0);
		}
		else
		{
			print "\nERROR: Selection not recognized. Please try again.\n";
		}
	}
}

##############################################################################
# run_ping
##############################################################################
sub run_ping {
	my($term) = shift(@_);
	my($host) = _get_info_from_terminal($term,"Hostname/IP");
	my($tries) = _get_info_from_terminal($term,"Number of times to ping (10)");
	$tries = 10 unless($tries =~ /^\d+$/);
	$host = strip_nasties($host);
	print "ping $host $tries times\n";
	print "\n=== ENTER CTRL+C to stop the ping before it completes ===\n\n";
	system("/bin/ping -c $tries $host");
	print "\n";
	my($ok) = _get_info_from_terminal($term,"Press Enter to return to Menu");
}

##############################################################################
# run_traceroute
##############################################################################
sub run_traceroute {
	my($term) = shift(@_);
	my($host) = _get_info_from_terminal($term,"Hostname/IP");
	$host = strip_nasties($host);
	print "\n=== ENTER CTRL+C to stop the traceroute before it completes ===\n\n";
	system("/bin/traceroute $host");
	print "\n";
	my($ok) = _get_info_from_terminal($term,"Press Enter to return to Menu");
}

##############################################################################
# show_resolv_conf
##############################################################################
sub show_resolv_conf {
	my($term) = shift(@_);
	my($file) = '/etc/resolv.conf';
	if(-f $file) {
		print "\n";
		open(RESOLV, "/etc/resolv.conf");
			while(<RESOLV>) {
				print $_;
			}
		close(RESOLV);
		print "\n";
	}
	else {
		print "ERROR: $file is missing!\n";
	}
	my($ok) = _get_info_from_terminal($term,"Press Enter to return to Menu");
}

##############################################################################
# primarify
##############################################################################
sub primarify
{
	return unless $bup_status->{is_backup};
	my $term = shift;

	my $validation = _get_info_from_terminal($term, "Are you sure you want to turn this server into the Primary Server?\ny/N ");
	return if $validation !~ /^y/i;

	_get_info_from_terminal($term, "
First unplug your current primary server from the internet.
Once done, press Enter");

	if (_alternate_is_up($bup_status->{'is_backup'}))
	{
		if (_get_info_from_terminal($term, "
We found your primary server is still reachable.
If you have not disconnected it from the internet
you may have issues with phone registration.

Do you wish to continue anyway?
y/N ") !~ /^[yY]/)
		{
			return 0;
		}
	}

	print "\nSecond, we are setting this server as Primary...\n";
	my $lwp_request = LWP::UserAgent->new();
	my ($server_id) = hostname() =~ /\A \D* (\d+) \z/xms;

	my ($user, $pass) = _get_user_pass();

	my $url = 'http://cp.' . ($server_id > 99999 ? 'trixbox' : 'fonality') . ".com/make_primary.cgi?user=$user&hash=$pass";
	my $ret = $lwp_request->get($url);
	print $ret->content(), "\n";

	_get_info_from_terminal($term, "
Please change the IP address of this server to be that of the primary server.
Press Enter to proceed");
	system('/usr/bin/perl /var/adm/bin/ip_set.pl -i');
	system('/usr/bin/perl /var/adm/bin/ping_main.pl');
	$bup_status = _get_bup_status();

	if ($bup_status->{'is_backup'})
	{
		_get_info_from_terminal($term, "
There was a failure while turning this server into the primary server.
Please contact support. Press Enter to return to the Menu");
		return 0;
	}
	else
	{
		print "Lastly, activating any VoIP accounts.\n";
		modify_voip_accounts('activate');

		_get_info_from_terminal($term, "Press Enter to return to the Menu");
		return 1;
	}
}

##############################################################################
# backupify
##############################################################################
sub backupify
{
	return unless $bup_status->{has_backup};
	my $term = shift;

	my $validation = _get_info_from_terminal($term, "Are you sure you want to turn this server into the Backup Server?\ny/N");
	return if $validation !~ /^y/i;

	print "First we are de-activating VoIP accounts.\n";
	modify_voip_accounts('deactivate');

	_net_stop();

	_get_info_from_terminal($term, "
The network has been stopped. Please plug in the network cable.
Once done, press Enter");

	_get_info_from_terminal($term, "
Please change the IP address of this server to be that of the backup server.
Press Enter to proceed");
	system('/usr/bin/perl /var/adm/bin/ip_set.pl -i');

	# ip_set.pl no longer starts the network for us
	_net_start();

	# Make sure to run ping_main, which will move the standby file to backup for us.
	system('/usr/bin/perl /var/adm/bin/ping_main.pl');
	$bup_status = _get_bup_status();

	if ($bup_status->{'has_backup'})
	{
		_get_info_from_terminal($term, "
There was a failure while turning this server into the backup server.
Please contact support. Press Enter to return to the Menu");
		return 0;
	}
	else
	{
		_get_info_from_terminal($term, "
This system is now the backup server.
Press Enter to return to the Menu");
		return 1;
	}
}

##############################################################################
# show_route
##############################################################################
sub show_route {
	my($term) = shift(@_);
	my($cmd) = '/bin/netstat';
	my($arg) = '-rn';
	if(-x $cmd) {
		print "\n";
		system("$cmd $arg");
		print "\n";
	}
	else {
		print "ERROR: $cmd is missing!\n";
	}
	my($ok) = _get_info_from_terminal($term,"Press Enter to return to Menu");
}

##############################################################################
# show_arp
##############################################################################
sub show_arp {
	my($term) = shift(@_);
	my($cmd) = '/sbin/arp';
	my($arg) = '-an';
	if(-x $cmd) {
		print "\n";
		system("$cmd $arg");
		print "\n";
	}
	else {
		print "ERROR: $cmd is missing!\n";
	}
	my($ok) = _get_info_from_terminal($term,"Press Enter to return to Menu");
}

##############################################################################
# show_ip
##############################################################################
sub show_ip {
	my($term) = shift(@_);
	my($cmd) = '/sbin/ifconfig';
	my($arg) = 'eth0';
	if(-x $cmd) {
		print "\n";
		system("$cmd $arg");
		print "\n";
	}
	else {
		print "ERROR: $cmd is missing!\n";
	}
	my($ok) = _get_info_from_terminal($term,"Press Enter to return to Menu");
}

##############################################################################
# _get_info_from_terminal
##############################################################################
sub _get_info_from_terminal {
    my($term) = shift(@_);
    my($prompt) = shift(@_) . ': ';
    my($answer) = $term->readline($prompt);
    $term->addhistory($_) if /\S/;
    return($answer);
}

##############################################################################
# _show_voltage
##############################################################################
sub _show_voltage {
    my $sangoma = 0;
    my $lspci = `lspci -vn`;

    $sangoma = 1 if $lspci =~ m/1923:0040/;

    my($term) = shift(@_);
	my($dir) = '/proc/zaptel';
	chdir($dir) || die "Can't chdir to $dir: $!";
	my($count) = 0;
	my($out);

	opendir(DIR, $dir);
	my(@card_list) = sort grep { /^\d/ } readdir(DIR);
	closedir(DIR);

	foreach my $card (@card_list) {
        my(@ports);
        $out .= "\nCard $card\n";
        open(FILE, $card) || die $!;
        while(<FILE>) {
                if(/\s+(\d+).*FXS.S/) {
                        push(@ports, $1);
                }
        }
        close(FILE);
        foreach my $port (@ports) {
                $out .= "\tPort $port\n" unless $sangoma;
                my(@lines) = `/usr/sbin/asterisk -r -x "zap show regdump $port"` unless $sangoma;
		my $sport = $port % 24;
		@lines = `wanpipemon -i w${card}g1 -c astats -m $sport` if $sangoma; 
                foreach my $line (@lines) {
                        if($line =~ /volt/i) {
                                $out .= "\t\t$line";
                        }
                }
				$count++;
        }
	}
	if($count > 0) {
		print "\n~8 VDC = On a call, ~48 VDC = Waiting for a call, ~200 VAC = Ringing\n\n";
		print "$out\n";;
		print "\nUse SHIFT+PageUp/PageDown to scroll up/down\n\n";
	}
	else {
		print "\nNo analog phone lines found.\n\n";
	}
	_get_info_from_terminal($term, 'Press Enter to return to menu');
}

##############################################################################
# _show_t1_status
##############################################################################
sub _show_t1_status {
    my($term) = shift(@_);
	my($dir) = '/proc/zaptel';
	chdir($dir) || die "Can't chdir to $dir: $!";
	my($count) = 0;
	my($out);

	opendir(DIR, $dir);
	my(@card_list) = sort grep { /^\d/ } readdir(DIR);
	closedir(DIR);

	foreach my $card (@card_list) {
        my(@ports);
        open(FILE, $card) || die $!;
        while(<FILE>) {
                if(/Span (\d+): WPT(\d+)\/(\d+) ".*" (\S+)/) {
					$out .= "\nSpan ${1}: WPT${2}/${3} $4";
                	if(/Span (\d+): WPT(\d+)\/(\d+) ".*" (\S+) (\S+)/) {
						$out .= "\tStatus: $5 ALARM";
					}
					else {
						$out .= "\tStatus: OK";
					}
					$out .= "\n";
					$count++;
                }
                elsif(/Span (\d+): TE4\/(\d+)\/(\d+) ".*" (\S+)/) {
					$out .= "\nSpan ${1}: WPT${2}/${3} $4";
                	if(/Span (\d+): TE4\/(\d+)\/(\d+) ".*" (\S+) (\S+)/) {
						$out .= "\tStatus: $5 ALARM";
					}
					else {
						$out .= "\tStatus: OK";
					}
					$out .= "\n";
					$count++;
                }
        }
        close(FILE);
	}
	if($count > 0) {
		print "$out\n";;
	}
	else {
		print "\nNo T1 spans found.\n\n";
	}
	_get_info_from_terminal($term, 'Press Enter to return to menu');
}

##############################################################################
# _set_root_password
##############################################################################
sub _set_root_password {
    my($term) = shift(@_);
	print "
You are about to set the root password for this system. Please do not proceed
unless you are an experienced Linux system administrator.\n\n";
	my($ans) = _get_info_from_terminal($term, 'Do you want to proceed? [y,n]');
	return(undef) unless($ans =~ /^y/i);
	system("/usr/bin/passwd root");
	_get_info_from_terminal($term, 'Press Enter to return to menu');
}

##############################################################################
# strip_nasties: Strip any bad characters out of a string
#
#    Args: text
# Returns: clean_text
##############################################################################
sub strip_nasties
{
	my $txt = shift;
	my $OK_CHARS ='-a-zA-Z0-9_.@';
	$txt =~ s/[^$OK_CHARS]/_/go;
	return $txt;
}

##############################################################################
# resend_email: Attempt to resend an e-mail to the admin of current box
#
#    Args: none
# Returns: nothing
##############################################################################
sub resend_email
{
	my $term = shift;

	my $lwp_request = LWP::UserAgent->new();
	my ($server_id) = hostname() =~ /\A \D* (\d+) \z/xms;

	my $url = "http://cp.trixbox.com/resend_email.cgi?server_id=$server_id";
	my $ret = $lwp_request->get($url);

	if ($ret->is_success())
	{
		print "Your e-mail was successfully resent.\n";
	}
	else
	{
		print "Your e-mail failed to send properly.\n";
	}

	_get_info_from_terminal($term, 'Press Enter to return to menu');
}

##############################################################################
# modify_voip_accounts: Run through sip.conf and iax.conf and make sure
#	the register lines are commented/not commented as appropriate.
#	This should be taken care of if the files sync down, as they should,
#	but in case they don't, let's be certain.
#
#    Args: none
# Returns: nothing
##############################################################################
sub modify_voip_accounts
{
	my $mode = shift;

	if ($mode eq 'activate')
	{
		_modify_voip('/etc/asterisk/iax.conf', 'uncomment');
		_modify_voip('/etc/asterisk/sip.conf', 'uncomment');
	}
	else
	{
		_modify_voip('/etc/asterisk/iax.conf', 'comment');
		_modify_voip('/etc/asterisk/sip.conf', 'comment');
	}
}

##############################################################################
# _modify_voip: comments/uncomments voip accounts in sip.conf or iax.conf
#
#    Args: file to parse, [comment/uncomment] (defaults to commenting)
# Returns: nothing
##############################################################################
sub _modify_voip
{
	my ($file, $mode) = @_;
	my $tmp_file = "/tmp.$$." . time;

	File::Copy::copy($file, $tmp_file);
	
	open my $bak, '<', $tmp_file or return;
	open my $new, '>', $file or do { close $bak; return; };

	while (my $line = <$bak>)
	{
		# For now just uncomment all register lines.
		# This logic needs to become more complex in the future. --jynx
		if ($line =~ /register/)
		{
			if ($mode eq 'uncomment')
			{
				$line =~ s/^;+//;
			}
			if ($mode eq 'comment')
			{
				$line = ';' . $line;
			}
		}

		print $new $line;
	}

	close $new;
	close $bak;

	unlink $tmp_file;

	return;
}

##############################################################################
# _get_user_pass: Get the username and password from /etc/asterisk/fonality/globals.conf
#
#    Args: none
# Returns: empty strings on failure, user and hashed pass on success
##############################################################################
sub _get_user_pass
{
	my ($user, $hash) = ('', '');

	open my $globals, '<', '/etc/asterisk/fonality/globals.conf' || return('','');
	while (my $line = <$globals>)
	{
		if ($line =~ /^IAXINFO/)
		{
			if ($line =~ /IAXINFO\d?=(s\d+):([^;\s]+)/)
			{
				($user, $hash) = ($1, Digest::MD5::md5_hex($2));
				$user =~ s/[xi]$//;
			}
			last;
		}
	}

	close $globals;
	return($user, $hash);
}

##############################################################################
# _get_bup_status: Get the username and password from /etc/asterisk/fonality/globals.conf
#
#    Args: none
# Returns: hash as follows:
#	{
#		part_of_pair => <boolean>,
#		is_backup    => <boolean>,
#		has_backup   => <boolean>,
#	}
##############################################################################
sub _get_bup_status
{
	my %ret;
	$ret{is_backup}    = -e '/etc/asterisk/primary' ? '/etc/asterisk/primary' : 0;
	$ret{has_backup}   = -e '/etc/asterisk/standby' ? '/etc/asterisk/standby' : -e '/etc/standby' ? '/etc/standby' : 0;
	$ret{part_of_pair} = $ret{is_backup} || $ret{has_backup} ? 1 : 0;

	return wantarray ? %ret : \%ret;
}

sub _alternate_is_up
{
	my $file = shift;
	return 0 unless $file;

	open my $fh, '<', $file or return 0;
	my $host = <$fh>;
	close $fh;

	chomp($host);

	my $ret = system( "/sbin/arping -c 1 \Q$host\E > /dev/null 2>&1" );
	my $rv = $ret ? 0 : 1;

	return $rv;
}

##############################################################################
# _net_stop: stop the network.
# Necessary for turning a server into a backup.
##############################################################################
sub _net_stop
{
	system("service vpn stop");
	system("/etc/init.d/network stop");
}

##############################################################################
# _net_start: (re)start the network
# In case ip_set.pl doesn't start the network, we need to make sure it's up.
# This might restart the network twice, but better that than not at all.
##############################################################################
sub _net_start
{
	system("/etc/init.d/network restart");
	system("service vpn restart");
}
