#!/usr/bin/perl 
#
# PBXtra Diagnosis Tool
# 
# This tool is developed to allow diagnosis of possible problems
# with a PBXtra system or network. It is not meant to discover the
# problems exactly, but to hone in on what could possibly be causing
# troubles with a system.
#
# Currently checks for:
#	Asterisk PID file
#	Asterisk running
#	Asterisk deadlocks
#	Bad tx/rxgain levels
#	HUD running (ircd and saxirc functioning properly, other HUD users)
#	Disk failures from RAID status
#	DHCP server accepting DISCOVERs and sending OFFERs
#	DHCP server returning name servers
#	Name servers set on the PBXtra machine
#	Verification that the DHCP name servers work (specifically the primary)
#	Verification that the system name servers work (specifically the primary)
#	Verification that our name server is accessable and works
#	Verifies cached IPs match the real IPs
#
# by Samy Kamkar, 03/16/05
#
# Copyright 2007, Fonality Inc. ALL RIGHTS RESERVED.

# constants for messages to display
use constant NOTICE => 1;	# should see all these
use constant WARNING => 2;	# should see all these
use constant ERROR => 4;	# should see all these
use constant DEBUG => 8;	# shouldn't always see these
use constant VERBOSE => 16;	# should hardly see these

use CPAN;
use strict;
use IO::Socket;
use Data::Dumper;
use Sys::Hostname;
use Term::ANSIColor qw/:constants/;
use ExtUtils::MakeMaker qw/prompt/;
use LWP::Simple();

$Term::ANSIColor::AUTORESET = 1;

my $getLease;

# get a lease?
if ($ARGV[0] =~ /^-l/)
{
	$getLease = shift(@ARGV);
}

# if we passed a server id
if ($ARGV[0] =~ /^\d+$/)
{
	print "Copying trouble.pl over to pbxtra$ARGV[0]...\n";
	system("scp $0 pbxtra$ARGV[0]:/var/adm/bin/");

	print "\nRunning trouble.pl on pbxtra$ARGV[0]...\n\n";

	my $file = $0;
	$file =~ s|^.*/([^/]+)$|$1|;

	system("ssh pbxtra$ARGV[0] perl /var/adm/bin/$file $getLease");
	exit;
}

# fake MAC address to use for DHCP discovery
my $FAKE_MAC = "00:90:27:17:c8:dc";

# default interface the net is on
my $INTERFACE = "eth0";

# max DHCP discovers to try before erroring
my $MAX_DHCP_DISCOVERS = 2;

# fonality's name server
my $FONALITY_NS = "pbxtra-ns1.fonality.com";

# only verify the primary name server works?
my $ONLY_PRIMARY_NS = 1;


# get the server ID
my $hostname = hostname();
(my $serverid) = ($hostname =~ /(\d+)/);

# other modules the system might not have but we need
my @MODULES = qw/
	Net::RawIP
	Net::DNS
	Net::DHCPClient
/;
my (@ERRORS, @WARNINGS);


# check we have the appropriate modules installed and install necessary ones
check_modules(@MODULES);

# verify we have Net::DHCPClient 1.2 (this is not a real version
# but is a modification to 1.0 by samyk)
check_dhcpclient();

# load all modules after we know we have them all installed
eval("use $_") foreach @MODULES;

# check tx/rxgains in zapata
check_gains();

# check hud
check_hud();

# check disks
check_disks();

# verify we have a valid hostname
check_hostname($hostname);

# verify asterisk is running
check_asterisk_running();

# verify asterisk is not deadlocked
check_asterisk_deadlock();

# find out if we can sync files by connecting back to us
check_www();

# verify DHCP is working
my $dhcpobj = check_dhcp();

# give out dhcp data
debug_dhcp($dhcpobj);

# verify DNS is working
check_dns($dhcpobj);

exit;


# verify whether DHCP is working, returns Net::DHCPClient object
sub check_dhcp
{
	debug(DEBUG, "Verifying DHCP server exists");
	debug(VERBOSE, "Creating a Net::DHCPClient object ($FAKE_MAC, $INTERFACE)");

	# Net::DHCPClient is stupid and closes STDERR and doesn't reopen it correctly
	open OLDERR, ">&", \*STDERR;
	my $a = new Net::DHCPClient(macaddr => $FAKE_MAC, interface => $INTERFACE, debug => 0, timeout => 10);

	(my $tmpmac = $FAKE_MAC) =~ s/:/ /g;
	my $i = 0;
	my $max = 0;
	while ($i == 0)
	{
		debug(DEBUG, "Sending a DHCP " . ($getLease ? "lease" : "discover"));

		# watch for timeouts
		eval
		{
			$SIG{ALRM} = sub { die };
			alarm(10);
			$i = ($getLease ? $a->request(61 => $tmpmac) : $a->discover(61 => $tmpmac));
			alarm(0);
		};

		# Net::DHCPClient kills STDERR, let's fix that!
		open STDERR, ">&OLDERR";

		# if we can't discover the DHCP server after 
		if (++$max == $MAX_DHCP_DISCOVERS && !$i)
		{
			if ($@)
			{
				debug(ERROR, "Unable to discover a DHCP server: timed out", << "EOF");
This can cause problems such as:
	The PBXtra system or phones not getting IPs after reboots unless statics are set
	The PBXtra system or phones not getting name servers for resolution
EOF
			}
			else
			{
				debug(ERROR, "Unable to discover a DHCP server", << "EOF");
This can cause problems such as:
	The PBXtra system or phones not getting IPs after reboots unless statics are set
	The PBXtra system or phones not getting name servers for resolution
EOF
			}

			last;
		}
		elsif ($i)
		{
			debug(DEBUG, "DHCP offer returned from: " . join(" ", $a->server_ip()));

			my @pcap = $a->{pcap}->get({ip => [qw(saddr daddr)]});
			debug(DEBUG, "DHCP returned IP lease to address: " . join(".", unpack("C4", pack("N", $pcap[1]))));
			debug(DEBUG, "DHCP gave following DNS server(s): " . join(" ", $a->domain_name_server()));
			debug(DEBUG, "DHCP gave following NTP server(s): " . join(" ", $a->ntp_server()));
			debug(DEBUG, "DHCP gave following time server(s): " . join(" ", $a->time_server()));
			debug(DEBUG, "DHCP gave following TFTP server(s): " . join(" ", $a->server_lookup(66)));
			debug(DEBUG, "DHCP gave following time offset: " . join(" ", $a->server_lookup(2)));
			debug(DEBUG, "DHCP gave following lease time: " .
				unpack("I", pack("C4", split(/\./, join("", $a->server_lookup(51))))));
		}
	}

	return $i ? $a : undef;
}


# querys the 'i', 'x' and default host, returning all records
sub query_all_hosts
{
	my ($nsobj, $ns) = @_;

	my %addys;

	# if we have a name server object
	if ($nsobj)
	{
		# set the name server
		$nsobj->nameservers($ns);

		# lookup our hosts
		foreach my $h ("i", "x", "")
		{
			my $host = "s${serverid}${h}.pbxtra.fonality.com";
			$addys{$host} = query($nsobj, $host);
		}

		return \%addys;
	}

	return undef;
}


# queries a single host returning an array ref of IPs
sub query
{
	my ($nsobj, $host) = @_;

	my @addys;

	# get a list of nameservesr we're using
	my $ns = join ",", $nsobj->nameservers();

	# query the host
	my $query = $nsobj->query($host, "A");

	# if we queried ok
	if ($query)
	{
		# go through each answer
		foreach my $rr ($query->answer())
		{
			# if it's an A record
			if ($rr->type eq 'A')
			{
				debug(VERBOSE, "Resolved $host to " . $rr->address() . " using $ns");
				push @addys, $rr->address();
			}
			# if it's a CNAME
			elsif ($rr->type eq 'CNAME')
			{
				debug(VERBOSE, "Resolved $host to " . $rr->cname() . " using $ns");
				push @addys, @{query($nsobj, $rr->cname())};
			}
		}
	}

	return \@addys;
}


# check disks (specifically raid)
sub check_disks
{
	debug(DEBUG, "Checking disks / RAID");
	unless (open(PROC, "</proc/mdstat"))
	{
		debug(ERROR, "Can't read /proc/mdstat: $!", << "EOF");
This contains information on RAID status and should exist even if the system doesn't have RAID!
EOF
		return 0;
	}

	my $disk;
	my @raids;
	my %disks;
	while (<PROC>)
	{
		if (/^Personalities : (.+)/)
		{
			@raids = map { s/[\[\]]//g; $_ } split(/\s+/, $1);
			debug(DEBUG, "Found raid types: @raids");
		}
		elsif (/^(md\d+) : (\S+) (\S+) (.*)/)
		{
			$disk = $1;
			$disks{$disk} = {
				status	=> $2,
				raid	=> $3,
				disks	=> [map { s/\[\d*\]//; $_ } split(" ", $4)],
			};
		}
		elsif (/^\s+\d+\s+\S+\s+\[(\d+)\/(\d+)\]\s+\[(.*)\]/)
		{
			my ($totaldisks, $activedisks, $status) = ($1, $2, $3);
			if ($totaldisks > $activedisks)
			{
				my $hdparm = `hdparm -vIi /dev/$disks{$disk}{disks}[0] 2>>/dev/null`;
				my ($gb) = ($hdparm =~ /\((\d+ GB)\)/);
				my ($model) = ($hdparm =~ /Model Number:\s+(\S+)/);


				my $string =	"You will want to probably RMA the drive with this disk:\n" .
						BOLD "\tModel: $model\n\tSize: $gb\n";

				debug(ERROR, "Drive failure in $disk (part of $disks{$disk}{raid})", $string, 1);

				$string =	"\n" . BOLD "/proc/mdstat:\n" . RESET `cat /proc/mdstat` .
						BOLD "\nmdadm --detail /dev/$disk\n" . RESET `mdadm --detail /dev/$disk`;

				debug(NOTICE, $string);

				last;
			}
		}
	}
	close(PROC);
}

# verify whether the DNS servers are working
sub check_dns
{
	my $dhcpobj = shift;
	my $nsobj = Net::DNS::Resolver->new();
	my @hosts;
	my @ns;

	unless ($nsobj)
	{
		debug(ERROR, "Can't create Net::DNS::Resolver object");
		return undef;
	}

	# get our IP according to our own NS
	debug(DEBUG, "Looking up DNS information from our remote DNS server");
	$hosts[0] = query_all_hosts($nsobj, $FONALITY_NS);
	if (ref($hosts[0]) eq "HASH")
	{
		foreach (keys %{$hosts[0]})
		{
			# if there are no hosts
			unless (@{$hosts[0]{$_}})
			{
				debug(ERROR, "Can't do a DNS query of $_ via $FONALITY_NS", << "EOF");
This shouldn't be a problem as long as the name servers on the network work.
EOF
			}
		}
	}
	else
	{
		debug(ERROR, "Couldn't query hosts through $FONALITY_NS: unknown reason");
	}


	# let's look for DNS servers from the DHCP server
	if ($dhcpobj)
	{
		# first check to see if the DNS servers from DHCP work
		debug(DEBUG, "Looking up DNS information from our DHCP DNS server(s)");
		@ns = $dhcpobj->domain_name_server();

		# if there's no name servers
		if (!@ns)
		{
			debug(ERROR, "DHCP server did not provide any name servers.", << "EOF");
This can cause troubles for any phones without name servers explicitly set on the phones.
EOF
		}

		else
		{
			verify_hosts($nsobj, \@hosts, [@ns]); # we don't want to modify @ns so we re-ref it
		}
	}
	else
	{
		debug(WARNING, "No DHCP server so cannot discover DNS servers from DHCP.");
	}


	# now try DNS servers in /etc/resolv.conf, but not ones we already tried
	debug(DEBUG, "Looking up DNS information from /etc/resolv.conf");
	if (open(RESOLV, "</etc/resolv.conf"))
	{
		my @newns;
		while (<RESOLV>)
		{
			if (/^nameserver\s+(\S+)/)
			{
				push @newns, $1;
			}
		}
		close(RESOLV);

		# if there's no name servers
		if (!@newns)
		{
			debug(ERROR, "/etc/resolv.conf server did not provide any name servers.", << "EOF");
This can cause troubles for any phones without name servers explicitly set on the phones.
EOF
		}

		else
		{
			for (my $i = 0; $i < @newns; $i++)
			{
				# if this name server in resolv.conf was also returned by DHCP
				if (grep { $_ eq $newns[$i] } @ns)
				{
					# delete it
					splice(@newns, $i--, 1);
					last;
				}
			}

			if (@newns)
			{
				verify_hosts($nsobj, \@hosts, [@newns]); # we don't want to modify @ns so we re-ref it
			}
		}
	}
	else
	{
		debug(ERROR, "Can't open /etc/resolv.conf for reading: $!", << "EOF");
This can cause possible system troubles such as:
	Not being able to talk to linked servers (/etc/asterisk/iax.conf)
	Not being able to connect tunnels (/etc/rc.local)
	Slowing down services such as Apache and ssh due to reverse name lookups
EOF
	}
}


# verify the local name servers lookup the IPs of the PBXtra machine
# to the same IPs that our name server has for them
sub verify_hosts
{
	my ($nsobj, $hosts, $ns) = @_;
	my @hosts = @{$hosts};
	my @ns = @{$ns};

	# if we only want to check the primary name server
	if ($ONLY_PRIMARY_NS)
	{
		@ns = $ns[0];
	}

	# check if each server works
	for (my $i = 0; $i < @ns; $i++)
	{
		# get our IP according to the NS from DHCP
		$hosts[$i+1] = query_all_hosts($nsobj, $ns[$i]);
		if (ref($hosts[$i+1]) eq "HASH")
		{
			foreach my $host (keys %{$hosts[$i+1]})
			{
				# if there are no hosts
				unless (@{$hosts[$i+1]{$host}})
				{
					if ($i)
					{
						debug(WARNING, "Can't do a DNS query of $host via secondary name server $ns[$i]", << "EOF");
This should not affect anything unless the primary name server is not functioning.
EOF
					}
					else
					{
						debug(ERROR, "Can't do a DNS query of $host via primary name server $ns[$i]", << "EOF");
This can cause numerous problems including:
	Phones not registering
	Phones not being able to make outbound calls
	Phones registering temporarily but not working properly or working intermittently
It can also possibly cause:
	Not being able to talk to linked servers (/etc/asterisk/iax.conf)
	Not being able to connect tunnels (/etc/rc.local)
	Slowing down services such as Apache and ssh due to reverse name lookups
EOF
					}
				}
				elsif (ref($hosts[0]) eq "HASH")
				{
					# go through all hosts and verify they resolve to the correct IPs
					foreach my $ip (@{$hosts[$i+1]{$host}})
					{
						unless (grep { $ip eq $_ } @{$hosts[0]{$host}})
						{
							debug(ERROR, "IPs resolved by $FONALITY_NS for $host do not match IPs resolved by $ns[0]", << "EOF");
This can possibly cause the following problems:
	Phones not to register and attempt to register to a cached IP
EOF
						}
					}
				}
			}
		}
		else
		{
			debug(ERROR, "Couldn't query hosts through $ns[$i]: unknown reason");		}
	}

}


# check for required modules for this to work
sub check_modules
{
	my @check = @_;
	my @notfound;

	foreach my $module (@check)
	{
		my $string;
		$string = substr("$module ............................", 0, 30);

		my $version = eval "use $module; \$${module}::VERSION";
		if ($version)
		{
			$string .= "found version $version";
			debug(VERBOSE, $string);
		}
		else
		{
			$string .= "** FAILED **";
			debug(WARNING, $string);

			push(@notfound, $module);
		}
	}

	# make sure we have the older version of Net::RawIP
	my $rawip = eval "use Net::RawIP; \$Net::RawIP::VERSION";
	if ($rawip eq "0.21" || grep { $_ eq "Net::RawIP" } @notfound)
	{
		chdir("/tmp");
		LWP::Simple::getstore("http://backpan.perl.org/authors/id/S/SK/SKOLYCHEV/Net-RawIP-0.2.tar.gz", "Net-RawIP-0.2.tar.gz");
		system("tar -zxf Net-RawIP-0.2.tar.gz && cd Net-RawIP-0.2 && perl Makefile.PL && make install");
		@notfound = grep { $_ ne "Net::RawIP" } @notfound;
	}

	foreach my $module (@notfound)
	{
		CPAN::Shell->install($module);
		print "\n";
	}
}



# debug function
sub debug
{
	my ($types, $message, $description, $noformat) = @_;

	if ($types & VERBOSE)
	{
		print "verbose: $message\n";
	}

	elsif ($types & DEBUG)
	{
		print "debug: $message\n";
	}

	elsif ($types & ERROR)
	{
		print BOLD RED "\n*** ERROR";
		print ": $message\n";
		if ($description)
		{
			if ($noformat)
			{
				print $description;
			}
			else
			{
				chomp($description);
				print map { s/^(?=\w)/  === /;  "$_\n" } split(/\n/, $description);
			}
		}
		print "\n";
	}

	elsif ($types & WARNING)
	{
		print BOLD YELLOW "WARNING";
		print ": $message\n";
	}

	elsif ($types & NOTICE)
	{
		print BOLD GREEN "NOTICE";
		print ": $message\n";
	}

}


# check for asterisk running
sub check_asterisk_running
{
	debug(DEBUG, "Checking that asterisk is running");

	# open the pid file
	if (open(PID, "</var/run/asterisk.pid"))
	{
		my $curpid;
		chomp(my $pid = <PID>);

		if ($pid !~ /^\d+$/)
		{
			debug(ERROR, "Asterisk PID file does not contain a PID", << "EOF");
/var/run/asterisk.pid does not contain a valid PID
Problems that can occur if Asterisk is not running:
	No inbound, outbound, or internal calls will work
	No phones will work
EOF
			return undef;
		}

		foreach (`ps auwx`)
		{
			# get the pid from the ps line
			my (undef, $curpid) = split();

			if ($curpid eq $pid)
			{
				$curpid = 0;
				last;
			}
		}

		# if we didn't find asterisk in ps output
		if ($curpid)
		{
			debug(ERROR, "Asterisk PID not found in `ps auwx` output", << "EOF");
Problems that can occur if Asterisk is not running:
	No inbound, outbound, or internal calls will work
	No phones will work
EOF
			return undef;
		}

		return $pid;
	}
	else
	{
		debug(ERROR, "Asterisk PID file not found", << "EOF");
/var/run/asterisk.pid not found
Problems that can occur if Asterisk is not running:
	No inbound, outbound, or internal calls will work
	No phones will work
EOF
	}
}


# check for asterisk deadlock
sub check_asterisk_deadlock
{
	debug(DEBUG, "Checking for asterisk deadlock");

	unless (`asterisk -r -x 'show channels'`)
	{
		debug(ERROR, "Asterisk is deadlocked", << "EOF");
`asterisk -r -x 'show channels'` did not return anything
Problems that can occur if Asterisk is deadlocked:
	Calls will work intermittently or have lots of static
	Much of the PBX functionality will only partially work
EOF
	}
}


# checks to make sure our hostname is the way it sould be
sub check_hostname
{
	my $hostname = shift;
	if ($hostname !~ /^pbxtra\d+$/)
	{
		debug(ERROR, "Hostname does not seem to be a valid PBXtra hostname", << "EOF");
This can cause possible problems such as:
	Specific logs not working
	Specific outbound or inbound calls not going through
This may also not be a valid PBXtra system.
EOF
	}
}


# check rxgain/txgains
sub check_gains
{
	unless (open(ZAPATA, "</etc/asterisk/zapata.conf"))
	{
		debug(ERROR, "Unable to read zapata.conf", << "EOF");
This can cause possible problems such as:
	Asterisk not starting!
EOF
		return 0;
	}

	while (<ZAPATA>)
	{
		if (/^\s*([tr])xgain\s*=\s*(\S+)/)
		{
			if ($2 >= 3 || $2 <= -3)
			{
				debug(ERROR, "${1}xgain in zapata.conf is way too low/high", << "EOF");
This can cause possible problems such as:
	Outbound calls not going out, or connecting and having no audio/DTMF.
	Incoming calls not coming in, or coming in and having no audio/DTMF.
Adjust the rxgains/txgains to a more sane level (-3 < gain < 3).
EOF
				return 0;
			}
		}
	}
	close(ZAPATA);
}

sub check_dhcpclient
{
	foreach my $file (map { "$_/Net/DHCPClient.pm" } @INC)
	{
		if (-e $file)
		{
			my @data;
			my $update;
			my $origver;

			open(FILE, "<$file");
			while (<FILE>)
			{
				if (s/Revision: (1\.[01])/Revision: 1.2/)
				{
					$origver = $1;
					$update = 1;
				}
				elsif (/Revision: 1\.2/)
				{
					last;
				}
				s/ether dst \$macaddr and dst port 68/dst port 68/;
				s/my \@g = /\$self->{pcap} = \$p;\n\tmy \@g = /;

				push @data, $_;
			}
			close(FILE);

			if ($update)
			{
				debug(VERBOSE, "Updating Net::DHCPClient from $origver to 1.2");
				open(FILE, ">$file");
				print FILE @data;
				close(FILE);

				eval("use Net::DHCPClient");
			}
		}
	}
}


sub debug_dhcp
{
	my ($c, $i) = @_;

	debug(DEBUG, "DHCP offer returned\n");

	my %commands;
	my $command;

	# generate hash of data
	while (<DATA>)
	{
		# normal line
		if (/^\s+(\d+)(?:-(\d+))?\s+(\S+.*?)(?:\s+(\d+|N)\s+(\S+.*?))?(?:\s+\[(.*?)\])?\s*$/)
		{
			my ($cmd1, $cmd2, $name, $len, $type, $rfc) = ($1, $2, $3, $4, $5, $6);

			$cmd2 ||= $cmd1;
			foreach ($cmd1 .. $cmd2)
			{

				$commands{$_} = {
					name => $name,
					length => $len,
					type => $type,
					rfc => $rfc,
				};
			}
			$command = $cmd2;
		}
		elsif (/^\s+([a-z].*)\s*$/i)
		{
			$commands{$command}{type} .= " $1";
		}
	}

	my %options = %{$c->options};
	my $noval;

	foreach my $opt (sort { $a <=> $b } keys %options)
	{

		if ($commands{$opt}{length} eq "0")
		{
			$noval = "\n";
		}
		else
		{
			$noval = "";
		}

		debug(DEBUG, "Option $opt: $commands{$opt}{name}");
		debug(DEBUG, "     type: $commands{$opt}{type} ($commands{$opt}{length} bytes: $commands{$opt}{rfc})$noval");

		if ($commands{$opt}{length} == 4 && $commands{$opt}{type} !~ /time$/i)
		{
			my @data = split(/ /, $options{$opt});
			while (@data != 4)
			{
				unshift(@data, 0);
			}

			debug(DEBUG, "    value: " . join(".", @data) . 
				" (dec: " . unpack("N", pack("C4", @data)) . ")\n");
		}
		elsif ($commands{$opt}{type} =~ /addresses$/i)
		{
			my @data = split(/ /, $options{$opt});
			my @ips;

			while (@data)
			{
				push @ips, join(".", splice(@data, 0, 4));
			}
			debug(DEBUG, "    value: " . join(", ", @ips) . "\n");
		}
		elsif ($commands{$opt}{type} =~ /name$/i || $commands{$opt}{name} =~ /name$/i)
		{
			debug(DEBUG, "    value: " . join("", map { chr($_) } split(/ /, $options{$opt})) . "\n");
		}
		elsif ($commands{$opt}{type} =~ /time$/i)
		{
			my @data = split(/ /, $options{$opt});
			if ($commands{$opt}{length} =~ /^\d+$/)
			{
				while (@data != $commands{$opt}{length})
				{
					unshift(@data, 0);
				}
			}

			debug(DEBUG, "    value: " . (unpack("N", pack("C4", @data)) / 60 / 60) . " hours (" .
				(unpack("N", pack("C4", @data)) / 60 / 60 / 24) . " days)\n");
		}
		elsif ($commands{$opt}{length} ne "0")
		{
			debug(DEBUG, "    value: $options{$opt}\n");
		}
	}
#	debug(DEBUG, "DHCP offer returned");

}

sub check_hud
{
	# if we have HUD installed
	debug(DEBUG, "Checking for HUD...");

	if (!-e "/usr/local/fonality/hud/auth.cfg")
	{
		debug(WARNING, "HUD configuration not found (HUD not installed?)");
		return 1;
	}

	# check for ircd/saxirc
	check_proc("ircd");
	check_proc("saxirc");

	# check for config file
	my $hudpass;
	unless (open(CONFIG, "</usr/local/fonality/hud/auth.cfg"))
	{
		debug(ERROR, "Couldn't read /usr/local/fonality/hud/auth.cfg: $!", << "EOF");
This can cause problems such as:
	HUD not working at all!
	More specifically, saxirc / ircd not running properly
EOF
		return 0;
	}
	else
	{
		chomp($hudpass = <CONFIG>);
		debug(DEBUG, "HUD password: $hudpass");
	}

	# try to IRC to localhost
	my $sock = IO::Socket::INET->new("localhost:6600");
	unless ($sock)
	{
		debug(ERROR, "Can't connect to HUD (ircd): localhost:6600", << "EOF");
This can cause problems such as:
	HUD not working at all!
	More specifically, ircd not working properly
EOF
		return 0;
	}

	print $sock "PASS $hudpass\nUSER test test test test\nNICK test333\n";
	my $ircdata;

	eval
	{
		local $SIG{ALRM} = sub { die "timeout\n" };
		alarm(3);

		while (<$sock>)
		{
			$ircdata .= "\t\t$_";
			chomp;

			# pings
			if (/^PING(.*)$/)
			{
				print $sock "PONG$1\n";
			}

			# we logged on
			elsif (/^\S+\s+MODE/)
			{
				print $sock "WHOIS Relay\n";
			}

			# we got a whois back
			elsif (/^\S+\s+311/)
			{
				debug(DEBUG, "Relay (saxirc) is properly connected to IRC");
				print $sock "JOIN #upgrade\n";
			}

			# we did NOT get a whois back
			elsif (/^\S+\s+401/)
			{
				debug(DEBUG, "Relay (saxirc) client is not logged into IRC! saxirc issue?");
			}

			# we got a names list back from joining a channel
			elsif (/^\S+\s+353\s+\S+\s+\S+\s+#upgrade\s+:(.*)$/)
			{
				my @names = split(/\s+/, $1);
				if (@names > 2)
				{
					debug(DEBUG, "Found " . @names . " users (including us and Relay) logged into HUD:\n" . 
						join "", map {
						s/[\@\+]//;
						print $sock "WHOIS $_\n";
						while (<$sock>)
						{
							if (/^\S+\s+311\s+\S+\s+(\S+)\s+\S+\s+(\S+)/)
							{
								"\t" . substr("$1          ", 0, 10) . ": $2\n"
							}
							last;
						}
					} @names);
				}
				else
				{
					debug(WARNING, "Found 0 other users logged into HUD (besides us and Relay) -- no other users logged into HUD?");
				}
			}
			last;
		}

		alarm(0);
	};
	if ($@ =~ /timeout/)
	{
		debug(ERROR, "Timed out while waiting for (or didn't receive proper) IRC response!", << "EOF");
IRC log:
$ircdata
EOF
	}
	elsif ($@)
	{
		debug(ERROR, $@);
	}
	close($sock);
}


sub check_proc
{
	my $proc = shift;
	debug(DEBUG, "Checking for $proc");

	my %ps = psgrep($proc);
	foreach (keys %ps)
	{
		$ps{$_} =~ s/\0//g;
		$ps{$_} =~ s/\s*$//g;
	}
	unless (%ps)
	{
		debug(ERROR, "Did not find $proc running!");
	}
}

sub check_www
{
	my $html = LWP::Simple::get("http://cp.fonality.com");
	if (index($html, "<html>") <= -1)
	{
		debug(ERROR, "Unable to reach http://cp.fonality.com syncing files won't work well $@");
	}
}

# safe (non-`ps`) ps + grep combo
# returns 1 if all params passed match a process command line
sub psgrep
{
        my (@grep) = @_;

        my %return;

        opendir(PROC, "/proc") || return undef;
        foreach my $pid (grep { /^\d+$/ } readdir(PROC))
        {
                open(CMD, "</proc/$pid/cmdline") || next;
                my $cmdline = <CMD>;
                close(CMD);

                # if everything matches     
                unless (grep { $cmdline !~ /$_/ } @grep)
                {
                        $cmdline =~ s/\0/ /g;     
                        $return{$pid} = $cmdline;
                }
        }
        closedir(PROC);

        return %return;          
}





__DATA__

    0      Pad                      0    None                                  [RFC2132]    
    1      Subnet Mask              4    Subnet Mask Value                     [RFC2132] 
    2      Time Offset              4    Time Offset in                        [RFC2132] 
                                         Seconds from UTC 
    3      Router                   N    N/4 Router addresses                  [RFC2132] 
    4      Time Server              N    N/4 Timeserver addresses              [RFC2132] 
    5      Name Server              N    N/4 IEN-116 Server addresses          [RFC2132] 
    6      Domain Server            N    N/4 DNS Server addresses              [RFC2132] 
    7      Log Server               N    N/4 Logging Server addresses          [RFC2132] 
    8      Quotes Server            N    N/4 Quotes Server addresses           [RFC2132] 
    9      LPR Server               N    N/4 Printer Server addresses          [RFC2132] 
   10      Impress Server           N    N/4 Impress Server addresses          [RFC2132] 
   11      RLP Server               N    N/4 RLP Server addresses              [RFC2132] 
   12      Hostname                 N    Hostname string                       [RFC2132] 
   13      Boot File Size           2    Size of boot file in 512 byte         [RFC2132] 
                                         chunks 
   14      Merit Dump File          N    Client to dump and name               [RFC2132] 
                                         the file to dump it to 
   15      Domain Name              N    The DNS domain name of the            [RFC2132] 
                                         client 
   16      Swap Server              N    Swap Server addeess                   [RFC2132] 
   17      Root Path                N    Path name for root disk               [RFC2132] 
   18      Extension File           N    Path name for more BOOTP info         [RFC2132] 
   19      Forward On/Off           1    Enable/Disable IP Forwarding          [RFC2132] 
   20      SrcRte On/Off            1    Enable/Disable Source Routing         [RFC2132] 
   21      Policy Filter            N    Routing Policy Filters                [RFC2132] 
   22      Max DG Assembly          2    Max Datagram Reassembly Size          [RFC2132] 
   23      Default IP TTL           1    Default IP Time to Live               [RFC2132] 
   24      MTU Timeout              4    Path MTU Aging Timeout                [RFC2132] 
   25      MTU Plateau              N    Path MTU  Plateau Table               [RFC2132] 
   26      MTU Interface            2    Interface MTU Size                    [RFC2132] 
   27      MTU Subnet               1    All Subnets are Local                 [RFC2132] 
   28      Broadcast Address        4    Broadcast Address                     [RFC2132] 
   29      Mask Discovery           1    Perform Mask Discovery                [RFC2132] 
   30      Mask Supplier            1    Provide Mask to Others                [RFC2132] 
   31      Router Discovery         1    Perform Router Discovery              [RFC2132] 
   32      Router Request           4    Router Solicitation Address           [RFC2132] 
   33      Static Route             N    Static Routing Table                  [RFC2132] 
   34      Trailers                 1    Trailer Encapsulation                 [RFC2132] 
   35      ARP Timeout              4    ARP Cache Timeout                     [RFC2132] 
   36      Ethernet                 1    Ethernet Encapsulation                [RFC2132] 
   37      Default TCP TTL          1    Default TCP Time to Live              [RFC2132] 
   38      Keepalive Time           4    TCP Keepalive Interval                [RFC2132] 
   39      Keepalive Data           1    TCP Keepalive Garbage                 [RFC2132] 
   40      NIS Domain               N    NIS Domain Name                       [RFC2132] 
   41      NIS Servers              N    NIS Server Addresses                  [RFC2132] 
   42      NTP Servers              N    NTP Server Addresses                  [RFC2132] 
   43      Vendor Specific          N    Vendor Specific Information           [RFC2132] 
   44      NETBIOS Name Srv         N    NETBIOS Name Servers                  [RFC2132] 
   45      NETBIOS Dist Srv         N    NETBIOS Datagram Distribution         [RFC2132] 
   46      NETBIOS Node Type        1    NETBIOS Node Type                     [RFC2132] 
   47      NETBIOS Scope            N    NETBIOS Scope                         [RFC2132] 
   48      X Window Font            N    X Window Font Server                  [RFC2132] 
   49      X Window Manager         N    X Window Display Manager              [RFC2132] 
   50      Address Request          4    Requested IP Address                  [RFC2132] 
   51      Address Time             4    IP Address Lease Time                 [RFC2132] 
   52      Overload                 1    Overload "sname" or "file"            [RFC2132] 
   53      DHCP Msg Type            1    DHCP Message Type                     [RFC2132] 
   54      DHCP Server Id           4    DHCP Server Identification            [RFC2132] 
   55      Parameter List           N    Parameter Request List                [RFC2132] 
   56      DHCP Message             N    DHCP Error Message                    [RFC2132] 
   57      DHCP Max Msg Size        2    DHCP Maximum Message Size             [RFC2132] 
   58      Renewal Time             4    DHCP Renewal (T1) Time                [RFC2132] 
   59      Rebinding Time           4    DHCP Rebinding (T2) Time              [RFC2132] 
   60      Class Id                 N    Class Identifier                      [RFC2132] 
   61      Client Id                N    Client Identifier                     [RFC2132] 
   62      Netware/IP Domain        N    Netware/IP Domain Name                [RFC2242] 
   63      Netware/IP Option        N    Netware/IP sub Options                [RFC2242] 
   64      NIS-Domain-Name          N    NIS+ v3 Client Domain Name            [RFC2132] 
   65      NIS-Server-Addr          N    NIS+ v3 Server Addresses              [RFC2132] 
   66      Server-Name              N    TFTP Server Name                      [RFC2132] 
   67      Bootfile-Name            N    Boot File Name                        [RFC2132] 
   68      Home-Agent-Addrs         N    Home Agent Addresses                  [RFC2132] 
   69      SMTP-Server              N    Simple Mail Server Addresses          [RFC2132] 
   70      POP3-Server              N    Post Office Server Addresses          [RFC2132] 
   71      NNTP-Server              N    Network News Server Addresses         [RFC2132] 
   72      WWW-Server               N    WWW Server Addresses                  [RFC2132] 
   73      Finger-Server            N    Finger Server Addresses               [RFC2132] 
   74      IRC-Server               N    Chat Server Addresses                 [RFC2132] 
   75      StreetTalk-Server        N    StreetTalk Server Addresses           [RFC2132] 
   76      STDA-Server              N    ST Directory Assist. Addresses        [RFC2132] 
   77      User-Class               N    User Class Information                [RFC3004] 
   78      Directory Agent          N    directory agent information           [RFC2610] 
   79      Service Scope            N    service location agent scope          [RFC2610] 
   80      Rapid Commit             0    Rapid Commit                          [RFC4039] 
   81      Client FQDN              N    Fully Qualified Domain Name           [RFC-ietf-dhc-fqdn-option-13.txt] 
   82      Relay Agent Information  N    Relay Agent Information               [RFC3046] 
   83      iSNS                     N    Internet Storage Name Service         [RFC4174]
   84      REMOVED/Unassigned                                                  [RFC3679]
   85      NDS Servers              N    Novell Directory Services             [RFC2241] 
   86      NDS Tree Name            N    Novell Directory Services             [RFC2241] 
   87      NDS Context              N    Novell Directory Services             [RFC2241] 
   88      BCMCS Controller Domain Name list                                   [RFC4280]
   89      BCMCS Controller IPv4 address option                                [RFC4280]
   90      Authentication           N    Authentication                        [RFC3118] 
   91      client-last-transaction-time option                                 [RFC4388]  
   92      associated-ip option                                                [RFC4388] 
   93      Client System            N    Client System Architecture            [RFC3679]  
   94      Client NDI               N    Client Network Device Interface       [RFC3679] 
   95      LDAP                     N    Lightweight Directory Access Protocol [RFC3679] 
   96      REMOVED/Unassigned                                                  [RFC3679] 
   97      UUID/GUID                N    UUID/GUID-based Client Identifier     [RFC3679] 
   98      User-Auth                N    Open Group's User Authentication      [RFC2485] 
   99      GEOCONF_CIVIC                                                       [RFC-ietf-geopriv-dhcp-civil-09.txt] 
   100     REMOVED/Unassigned                                                  [RFC3679]
   101     REMOVED/Unassigned                                                  [RFC3679] 
   102-107 REMOVED/Unassigned                                                  [RFC3679]
   108     REMOVED/Unassigned                                                  [RFC3679]
   109     Unassigned                                                          [RFC3679]
   110     REMOVED/Unassigned                                                  [RFC3679]
   111     Unassigned                                                          [RFC3679]
   112     Netinfo Address          N    NetInfo Parent Server Address         [RFC3679] 
   113     Netinfo Tag              N    NetInfo Parent Server Tag             [RFC3679] 
   114     URL                      N    URL                                   [RFC3679]
   115     REMOVED/Unassigned                                                  [RFC3679] 
   116     Auto-Config              N    DHCP Auto-Configuration               [RFC2563] 
   117     Name Service Search      N    Name Service Search                   [RFC2937] 
   118     Subnet Selection Option  4    Subnet Selection Option               [RFC3011] 
   119     Domain Search            N    DNS domain serach list                [RFC3397] 
   120     SIP Servers DHCP Option  N    SIP Servers DHCP Option               [RFC3361] 
   121     Classless Static Route   N    Classless Static Route Option         [RFC3442] 
           Option 
   122     CCC                      N    CableLabs Client Configuration        [RFC3495]
   123     GeoConf Option          16    GeoConf Option                        [RFC3825]
   124     V-I Vendor Class              Vendor-Identifying Vendor Class       [RFC3925]
   125     V-I Vendor-Specific           Vendor-Identifying Vendor-Specific    [RFC3925]
           Information                   Information
   126     Removed/Unassigned                                                  [RFC3679]
   127     Removed/Unassigned                                                  [RFC3679]
   128     PXE - undefined (vendor specific) (Tentatively Assigned - June 23, 2005)
   128     Etherboot signature. 6 bytes: E4:45:74:68:00:00	
   128     DOCSIS "full security" server IP address	
   128     TFTP Server IP address (for IP Phone software load)	
   129     PXE - undefined (vendor specific) (Tentatively Assigned - 23 Jun 2005)
   129     Kernel options. Variable length string	
   129     Call Server IP address 	
   130     PXE - undefined (vendor specific) (Tentatively Assigned - 23 Jun 2005)
   130     Ethernet interface. Variable length string.	
   130     Discrimination string (to identify vendor)	
   131     PXE - undefined (vendor specific) (Tentatively Assigned - 23 Jun 2005)
   131     Remote statistics server IP address 	
   132     PXE - undefined (vendor specific) (Tentatively Assigned - 23 Jun 2005)
   132     802.1P VLAN ID 	
   133     PXE - undefined (vendor specific) (Tentatively Assigned - 23 Jun 2005)
   133     802.1Q L2 Priority	
   134     PXE - undefined (vendor specific) (Tentatively Assigned - 23 Jun 2005)
   134     Diffserv Code Point	
   135     PXE - undefined (vendor specific) (Tentatively Assigned - 23 Jun 2005)
   135     HTTP Proxy for phone-specific applications	
   136-149 Unassigned                                                           [RFC3942]
   150     TFTP server address (Tentatively Assigned - 23 Jun 2005)
   150     Etherboot	
   150     GRUB configuration path name	
   151-174 Unassigned                                                           [RFC3942]
   175     Etherboot (Tentatively Assigned - 23 Jun 2005)
   176     IP Telephone (Tentatively Assigned - 23 Jun 2005)
   177     Etherboot (Tentatively Assigned - 23 Jun 2005)
   177     PacketCable and CableHome (replaced by 122)	
   178-207 Unassigned                                                           [RFC3942]
   208     pxelinux.magic (string) = F1:00:74:7E (241.0.116.126) (Tentatively
             Assigned - 23 Jun 2005)
   209     pxelinux.configfile (text) (Tentatively Assigned - 23 Jun 2005)
   210     pxelinux.pathprefix (text) (Tentatively Assigned - 23 Jun 2005)
   211     pxelinux.reboottime (unsigned integer 32 bits) (Tentatively Assigned
             - 23 Jun 2005)
   212-219 Unassigned
   220     Subnet Allocation Option (Tentatively Assigned - 23 Jun 2005)
   221     Virtual Subnet Selection Option	(Tentatively Assigned - 23 Jun 2005)
   222-223 Unassigned                                                           [RFC3942]
   224-254 Private Use 
   255     End                      0    None                                  [RFC2132] 




